@@ -29,11 +29,12 @@ struct http_hdr;
  *Nothing to see with the RFC.
  */
 enum {
-	QPACK_ERR_NONE = 0,
-	QPACK_ERR_RIC,
-	QPACK_ERR_DB,
-	QPACK_ERR_TRUNCATED,
-	QPACK_ERR_HUFFMAN,
+	QPACK_ERR_NONE = 0,  /* no error */
+	QPACK_ERR_RIC,       /* cannot decode Required Insert Count prefix field */
+	QPACK_ERR_DB,        /* cannot decode Delta Base prefix field */
+	QPACK_ERR_TRUNCATED, /* truncated stream */
+	QPACK_ERR_HUFFMAN,   /* huffman decoding error */
+	QPACK_ERR_TOO_LARGE, /* decoded request/response is too large */
 };
 
 struct qpack_dec {
@@ -44,7 +45,7 @@ struct qpack_dec {
 };
 
 int qpack_decode_fs(const unsigned char *buf, uint64_t len, struct buffer *tmp,
-                    struct http_hdr *list);
+                    struct http_hdr *list, int list_size);
 int qpack_decode_enc(struct qcs *qcs, void *ctx);
 int qpack_decode_dec(struct qcs *qcs, void *ctx);
 
