@@ -33,7 +33,7 @@ void ring_free(struct ring *ring);
 ssize_t ring_write(struct ring *ring, size_t maxlen, const struct ist pfx[], size_t npfx, const struct ist msg[], size_t nmsg);
 int ring_attach(struct ring *ring);
 void ring_detach_appctx(struct ring *ring, struct appctx *appctx, size_t ofs);
-int ring_attach_cli(struct ring *ring, struct appctx *appctx);
+int ring_attach_cli(struct ring *ring, struct appctx *appctx, uint flags);
 int cli_io_handler_show_ring(struct appctx *appctx);
 void cli_io_release_show_ring(struct appctx *appctx);
 
