@@ -700,11 +700,12 @@ static void qcs_destroy(struct qcs *qcs)
 
 static inline int qcc_is_dead(const struct qcc *qcc)
 {
-	if (qcc->app_ops && qcc->app_ops->is_active &&
-	    qcc->app_ops->is_active(qcc, qcc->ctx))
-		return 0;
-
-	if ((qcc->conn->flags & CO_FL_ERROR) || !qcc->task)
+	/* Mux connection is considered dead if :
+	 * - all stream-desc are detached AND
+	 *   = connection is on error OR
+	 *   = mux timeout has already fired or is unset
+	 */
+	if (!qcc->nb_sc && ((qcc->conn->flags & CO_FL_ERROR) || !qcc->task))
 		return 1;
 
 	return 0;
@@ -914,7 +915,7 @@ static int qcs_build_stream_frm(struct qcs *qcs, struct buffer *out, char fin,
 	return -1;
 }
 
-/* Check after transfering data from qcs.tx.buf if FIN must be set on the next
+/* Check after transferring data from qcs.tx.buf if FIN must be set on the next
  * STREAM frame for <qcs>.
  *
  * Returns true if FIN must be set else false.
@@ -979,22 +980,6 @@ void qcc_streams_sent_done(struct qcs *qcs, uint64_t data, uint64_t offset)
  */
 static int qc_send_frames(struct qcc *qcc, struct list *frms)
 {
-	/* TODO implement an opportunistic retry mechanism. This is needed
-	 * because qc_send_app_pkts is not completed. It will only prepare data
-	 * up to its Tx buffer. The frames left are not send even if the Tx
-	 * buffer is emptied by the sendto call.
-	 *
-	 * To overcome this, we call repeatedly qc_send_app_pkts until we
-	 * detect that the transport layer has send nothing. This could happen
-	 * on congestion or sendto syscall error.
-	 *
-	 * When qc_send_app_pkts is improved to handle retry by itself, we can
-	 * remove the looping from the MUX.
-	 */
-	struct quic_frame *first_frm;
-	uint64_t first_offset = 0;
-	char first_stream_frame_type;
-
 	TRACE_ENTER(QMUX_EV_QCC_SEND, qcc->conn);
 
 	if (LIST_ISEMPTY(frms)) {
@@ -1004,34 +989,7 @@ static int qc_send_frames(struct qcc *qcc, struct list *frms)
 
 	LIST_INIT(&qcc->send_retry_list);
 
- retry_send:
-	first_frm = LIST_ELEM(frms->n, struct quic_frame *, list);
-	if ((first_frm->type & QUIC_FT_STREAM_8) == QUIC_FT_STREAM_8) {
-		first_offset = first_frm->stream.offset.key;
-		first_stream_frame_type = 1;
-	}
-	else {
-		first_stream_frame_type = 0;
-	}
-
-	if (!LIST_ISEMPTY(frms))
-		qc_send_app_pkts(qcc->conn->handle.qc, 0, frms);
-
-	/* If there is frames left, check if the transport layer has send some
-	 * data or is blocked.
-	 */
-	if (!LIST_ISEMPTY(frms)) {
-		if (first_frm != LIST_ELEM(frms->n, struct quic_frame *, list))
-			goto retry_send;
-
-		/* If the first frame is STREAM, check if its offset has
-		 * changed.
-		 */
-		if (first_stream_frame_type &&
-		    first_offset != LIST_ELEM(frms->n, struct quic_frame *, list)->stream.offset.key) {
-			goto retry_send;
-		}
-	}
+	qc_send_app_pkts(qcc->conn->handle.qc, 0, frms);
 
 	/* If there is frames left at this stage, transport layer is blocked.
 	 * Subscribe on it to retry later.
@@ -1090,7 +1048,7 @@ static int _qc_send_qcs(struct qcs *qcs, struct list *frms)
 	/* out buffer cannot be emptied if qcs offsets differ. */
 	BUG_ON(!b_data(out) && qcs->tx.sent_offset != qcs->tx.offset);
 
-	/* FIN is set if all incoming data were transfered. */
+	/* FIN is set if all incoming data were transferred. */
 	fin = qcs_stream_fin(qcs);
 
 	/* Build a new STREAM frame with <out> buffer. */
@@ -1289,6 +1247,7 @@ static struct task *qc_io_cb(struct task *t, void *ctx, unsigned int status)
 	if (qc_release_detached_streams(qcc)) {
 		if (qcc_is_dead(qcc)) {
 			qc_release(qcc);
+			goto end;
 		}
 		else if (qcc->task) {
 			if (qcc_may_expire(qcc))
@@ -1301,6 +1260,7 @@ static struct task *qc_io_cb(struct task *t, void *ctx, unsigned int status)
 
 	qc_recv(qcc);
 
+ end:
 	TRACE_LEAVE(QMUX_EV_QCC_WAKE);
 
 	return NULL;
@@ -1516,6 +1476,10 @@ static size_t qc_rcv_buf(struct stconn *sc, struct buffer *buf,
 
 	cs_htx = htx_from_buf(buf);
 	if (htx_is_empty(cs_htx) && htx_used_space(qcs_htx) <= count) {
+		/* EOM will be copied to cs_htx via b_xfer(). */
+		if (qcs_htx->flags & HTX_FL_EOM)
+			fin = 1;
+
 		htx_to_buf(cs_htx, buf);
 		htx_to_buf(qcs_htx, &qcs->rx.app_buf);
 		b_xfer(buf, &qcs->rx.app_buf, b_data(&qcs->rx.app_buf));
@@ -1545,6 +1509,7 @@ static size_t qc_rcv_buf(struct stconn *sc, struct buffer *buf,
 		if (se_fl_test(qcs->sd, SE_FL_ERR_PENDING))
 			se_fl_set(qcs->sd, SE_FL_ERROR);
 
+		/* Set end-of-input if FIN received and all data extracted. */
 		if (fin)
 			se_fl_set(qcs->sd, SE_FL_EOI);
 
